import { Inject, Injectable, Logger } from '@nestjs/common';
import { EventEmitter2 } from '@nestjs/event-emitter';
import { AdEntity } from '../core/domain/ad.entity';
import { AdMapper } from '../ad.mapper';
import { AdRepositoryPort } from '../core/application/ports/ad.repository.port';
import {
  LoggerBase,
  MessagePublisherPort,
  PrismaRepositoryBase,
} from '@mobicoop/ddd-library';
import { PrismaService } from './prisma.service';
import { AD_MESSAGE_PUBLISHER } from '../ad.di-tokens';

export type AdBaseModel = {
  uuid: string;
  userUuid: string;
  driver: boolean;
  passenger: boolean;
  frequency: string;
  fromDate: Date;
  toDate: Date;
  monTime: Date;
  tueTime: Date;
  wedTime: Date;
  thuTime: Date;
  friTime: Date;
  satTime: Date;
  sunTime: Date;
  monMargin: number;
  tueMargin: number;
  wedMargin: number;
  thuMargin: number;
  friMargin: number;
  satMargin: number;
  sunMargin: number;
  seatsProposed: number;
  seatsRequested: number;
  strict: boolean;
  createdAt: Date;
  updatedAt: Date;
};

export type AdReadModel = AdBaseModel & {
  waypoints: WaypointModel[];
};

export type AdWriteModel = AdBaseModel & {
  waypoints: {
    create: WaypointModel[];
  };
};

export type WaypointModel = {
  uuid: string;
  position: number;
  lon: number;
  lat: number;
  name?: string;
  houseNumber?: string;
  street?: string;
  locality?: string;
  postalCode?: string;
  country: string;
  createdAt: Date;
  updatedAt: Date;
};

/**
 *  Repository is used for retrieving/saving domain entities
 * */
@Injectable()
export class AdRepository
  extends PrismaRepositoryBase<AdEntity, AdReadModel, AdWriteModel>
  implements AdRepositoryPort
{
  constructor(
    prisma: PrismaService,
    mapper: AdMapper,
    eventEmitter: EventEmitter2,
    @Inject(AD_MESSAGE_PUBLISHER)
    protected readonly messagePublisher: MessagePublisherPort,
  ) {
    super(
      prisma.ad,
      prisma,
      mapper,
      eventEmitter,
      new LoggerBase({
        logger: new Logger(AdRepository.name),
        domain: 'ad',
        messagePublisher,
      }),
    );
  }
}
